'use strict';

const assert = require('assert');
const restify = require('restify');
const request = require('supertest');
const urlSemverParser = require('.');

const server = restify.createServer();

server.pre(urlSemverParser);

function checkBodyResponse(path, message) {
  return done => {
    request(server).get(path).expect(200, (err, res) => {
      if (err) {
        return done(err);
      }
      assert.equal(res.body.message, message);
      done();
    });
  };
}

function getRequestResponse(message) {
  return (req, res) => {
    res.send({ message });
  };
}

describe('Url Semver Parser', function() {
  before(function() {
    server.get(
      { path: '/', version: '1.2.3' },
      getRequestResponse('accept-version: 1.2.3')
    );

    server.get(
      { path: '/foo', version: '1.2.3' },
      getRequestResponse('accept-version: 1.2.3')
    );

    server.get(
      { path: '/foo', version: '1.0.0-alpha' },
      getRequestResponse('accept-version: 1.0.0-alpha')
    );

    server.get(
      { path: '/foo', version: '1.0.0-alpha+001' },
      getRequestResponse('accept-version: 1.0.0-alpha')
    );
  });

  it('should return a valid version in format X.Y.Z when at end of url',
    checkBodyResponse('/v1.2.3', 'accept-version: 1.2.3')
  );

  it(
    'should return a valid version in format X.Y.Z when nested',
    checkBodyResponse('/v1.2.3/foo', 'accept-version: 1.2.3')
  );

  it(
    'should return a valid pre-release version',
    checkBodyResponse('/v1.0.0-alpha/foo', 'accept-version: 1.0.0-alpha')
  );

  it(
    'should return a valid version with build meta data excluded',
    checkBodyResponse('/v1.0.0-alpha+001/foo', 'accept-version: 1.0.0-alpha')
  );

  it('should return status code for bad request', function(done) {
    request(server).get('/foo').expect(400, done);
  });

});
